package com.minemaarten.signals.rail.network.mc;

import io.netty.buffer.ByteBuf;

import java.util.stream.Stream;

import javax.annotation.Nullable;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.DimensionManager;

import com.minemaarten.signals.lib.HeadingUtils;
import com.minemaarten.signals.rail.network.EnumHeading;
import com.minemaarten.signals.rail.network.IPosition;

public class MCPos implements IPosition<MCPos>{

    private final BlockPos pos;
    private final int dimID;

    public MCPos(World world, BlockPos pos){
        this(world.field_73011_w.getDimension(), pos);
    }

    public MCPos(int dimID, BlockPos pos){
        this.dimID = dimID;
        this.pos = pos;
    }

    public MCPos(NBTTagCompound tag){
        this.dimID = tag.func_74762_e("d");
        this.pos = new BlockPos(tag.func_74762_e("x"), tag.func_74762_e("y"), tag.func_74762_e("z"));
    }

    public MCPos(ByteBuf b){
        this.dimID = b.readInt();
        this.pos = new BlockPos(b.readInt(), b.readInt(), b.readInt());
    }

    public void writeToNBT(NBTTagCompound tag){
        tag.func_74768_a("d", dimID);
        tag.func_74768_a("x", pos.func_177958_n());
        tag.func_74768_a("y", pos.func_177956_o());
        tag.func_74768_a("z", pos.func_177952_p());
    }

    public void writeToBuf(ByteBuf b){
        b.writeInt(dimID);
        b.writeInt(pos.func_177958_n());
        b.writeInt(pos.func_177956_o());
        b.writeInt(pos.func_177952_p());
    }

    @Nullable
    public World getWorld(){
        return DimensionManager.getWorld(dimID);
    }

    public BlockPos getPos(){
        return pos;
    }

    public int getX(){
        return pos.func_177958_n();
    }

    public int getY(){
        return pos.func_177956_o();
    }

    public int getZ(){
        return pos.func_177952_p();
    }

    public int getDimID(){
        return dimID;
    }

    public TileEntity getLoadedTileEntity(){
        World world = getWorld();
        return world != null && world.func_175667_e(getPos()) ? world.func_175625_s(getPos()) : null;
    }

    @Override
    public int compareTo(MCPos o){
        int xComp = Integer.compare(pos.func_177958_n(), o.pos.func_177958_n());
        if(xComp != 0) return xComp;
        int yComp = Integer.compare(pos.func_177956_o(), o.pos.func_177956_o());
        if(yComp != 0) return yComp;
        int zComp = Integer.compare(pos.func_177952_p(), o.pos.func_177952_p());
        if(zComp != 0) return zComp;
        return Integer.compare(dimID, o.dimID);
    }

    @Override
    public double distanceSq(MCPos other){
        if(dimID != other.dimID) {
            return 0; //When in different dimensions, we don't know the distance, but assume none, so the AStar algorithm will keep searching.
        } else {
            return pos.func_177951_i(other.pos);
        }
    }

    @Override
    public EnumHeading getRelativeHeading(MCPos from){
        if(dimID != from.dimID) return null;
        int xDiff = pos.func_177958_n() - from.pos.func_177958_n();
        int zDiff = pos.func_177952_p() - from.pos.func_177952_p();
        if(zDiff == 0) {
            if(xDiff == 1) return EnumHeading.EAST;
            if(xDiff == -1) return EnumHeading.WEST;
        } else if(xDiff == 0) {
            if(zDiff == 1) return EnumHeading.SOUTH;
            if(zDiff == -1) return EnumHeading.NORTH;
        }
        return null;
    }

    @Override
    public MCPos offset(EnumHeading heading){
        return offset(HeadingUtils.toFacing(heading));
    }

    public MCPos offset(EnumFacing facing){
        return new MCPos(dimID, pos.func_177972_a(facing));
    }

    @Override
    public Stream<MCPos> allHorizontalNeighbors(){
        return EnumHeading.valuesStream().map(this::offset);
    }

    @Override
    public boolean equals(Object obj){
        if(obj instanceof MCPos) {
            MCPos other = (MCPos)obj;
            return other.dimID == dimID && other.pos.equals(pos);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode(){
        return pos.hashCode() * 13 + dimID;
    }

    @Override
    public String toString(){
        return "(" + pos + ", dim:" + dimID + ")";
    }
}
